@@ -7,6 +7,7 @@ from copy import copy, deepcopy
 from anytree import NodeMixin, RenderTree, PreOrderIter, LevelOrderIter
 
 from logics.classes.propositional import Formula
+from logics.classes.errors import ErrorCode, CorrectionError
 
 
 class TableauxNode(NodeMixin):
@@ -551,7 +552,7 @@ class TableauxSystem:
                 return False
             return False, subst_dict
 
-    def is_correct_tree(self, tree, inference=None, return_error_list=False, parser=None):
+    def is_correct_tree(self, tree, inference=None, return_error_list=False, exit_on_first_error=False, parser=None):
         """Checks if a given tableaux (a node and its descendants) is correctly derived, given the rules of the system.
 
         Parameters
@@ -563,9 +564,11 @@ class TableauxSystem:
             with no justification are either premises or the negation of the conclusion (this behavior can be overriden
             for other systems, see the source code).
         return_error_list: bool, optional
-            If False, will just return True or False (exits when it finds an error, more efficient). If True, will
-            return a tuple (boolean, [error_list]) (computes all errors, does not exit on the first, less efficient).
-            The error list contains tuples (index path to node, str), see the examples below.
+            If False, will just return True or False (exits when it finds an error, more efficient) If True, will return
+            (True, a list of ``logics.classes.errors.CorrectionError``)
+        exit_on_first_error: bool, optional
+            If `return_error_list` and this are both true, it will return a list with a single error instead of many.
+            More efficient, since it makes early exits.
         parser: logics.utils.parsers.standard_parser.StandardParser, optional
             If present, will return the error list with unparsed instead of parsed formulae.
             Can be of another class that implements ``unparse`` for ``Formula``.
@@ -597,11 +600,11 @@ class TableauxSystem:
         False
         >>> classical_tableaux_system.is_correct_tree(n1, inference=classical_parser.parse('~~p ∧ q / ~p'),
         ...                                           return_error_list=True)
-        (False, [((0, 0), "Node ['~', ['p']] is an incorrect premise node"), ((), "Negation of conclusion ['~', ['p']] is not present in the tree")])
+        (False, [(): Conclusion ['~', ['p']] is not present in the tree, (0, 0): Node ['~', ['p']] is an incorrect premise node])
         >>> from logics.utils.parsers import classical_parser
         >>> classical_tableaux_system.is_correct_tree(n1, inference=classical_parser.parse('~~p ∧ q / ~p'),
         ...                                           return_error_list=True, parser=classical_parser)
-        (False, [((0, 0), 'Node ~p is an incorrect premise node'), ((), 'Negation of conclusion ~p is not present in the tree')])
+        (False, [(): Conclusion ~p is not present in the tree, (0, 0): Node ~p is an incorrect premise node])
 
         Notes
         -----
@@ -633,9 +636,13 @@ class TableauxSystem:
                 if not traversing_premises:
                     if not return_error_list:
                         return False
-                    error_list.append((tuple(n.child_index for n in node.path),
-                                       f'Premise nodes must be at the beggining of the tableaux, '
-                                       f'before applying any rule and before opening any new branch'))
+                    error_list.append(CorrectionError(code=ErrorCode.TBL_PREMISE_NOT_BEGINNING, category="TBL",
+                                                      index=tuple(n.child_index for n in node.path),
+                                                      description='Premise nodes must be at the beggining of the '
+                                                                  'tableaux, before applying any rule and before '
+                                                                  'opening any new branch'))
+                    if exit_on_first_error:
+                        return False, error_list
 
                 # If an inference was given
                 if inference is not None:
@@ -648,8 +655,12 @@ class TableauxSystem:
                     else:
                         if not return_error_list:
                             return False
-                        error_list.append((tuple(n.child_index for n in node.path),
-                                           f'Node {node._self_string(parser)} is an incorrect premise node'))
+                        error_list.append(CorrectionError(code=ErrorCode.TBL_INCORRECT_PREMISE, category="TBL",
+                                                          index=tuple(n.child_index for n in node.path),
+                                                          description=f'Node {node._self_string(parser)} is an '
+                                                                      f'incorrect premise node'))
+                        if exit_on_first_error:
+                            return False, error_list
                 else:
                     correctly_derived_nodes.add(node)  # Not really necessary but leave it just in case
             if len(node.children) > 1:
@@ -673,8 +684,12 @@ class TableauxSystem:
                     if not correct:
                         if not return_error_list:
                             return False
-                        error_list.append((tuple(n.child_index for n in node.path),
-                                           f'Rule {rule_name} was not applied to node {node._self_string(parser)}'))
+                        error_list.append(CorrectionError(code=ErrorCode.TBL_RULE_NOT_APPLIED, category="TBL",
+                                                          index=tuple(n.child_index for n in node.path),
+                                                          description=f'Rule {rule_name} was not applied to '
+                                                                      f'node {node._self_string(parser)}'))
+                        if exit_on_first_error:
+                            return False, error_list
                     else:
                         correctly_derived_nodes |= result3[1]
 
@@ -682,9 +697,12 @@ class TableauxSystem:
         if inference is not None:
             # returns a bool, MUTATES ERROR_LIST
             all_premises_present = self._all_premises_present(inference, present_premises, present_conclusions,
-                                                              return_error_list, error_list, parser)
-            if not all_premises_present and not return_error_list:
-                return False
+                                                              return_error_list, error_list, exit_on_first_error, parser)
+            if not all_premises_present:
+                if not return_error_list:
+                    return False
+                if exit_on_first_error:
+                    return False, error_list
 
         # After checking all nodes and all rules that can be applied to it, the correctly derived nodes should be
         # all the tree nodes (that are not premises),
@@ -695,15 +713,19 @@ class TableauxSystem:
             if not return_error_list:
                 return False
             for node in unaccounted_nodes:
-                error_list.append((tuple(n.child_index for n in node.path),
-                                   f'Rule incorrectly applied in node {node._self_string(parser)}'))
+                error_list.append(CorrectionError(code=ErrorCode.TBL_RULE_INCORRECTLY_APPLIED, category="TBL",
+                                                  index=tuple(n.child_index for n in node.path),
+                                                  description=f'Rule incorrectly applied to '
+                                                              f'node {node._self_string(parser)}'))
+                if exit_on_first_error:
+                    return False, error_list
 
         # If you got to here with return_error_list = False, then everything is alright
         if not return_error_list:
             return True
         # Oterwise, check if there are errors
         if error_list:
-            return False, sorted(error_list, key=lambda e: (len(e[0]), e[0]))  # errors sorted by LevelOrder
+            return False, sorted(error_list, key=lambda e: (len(e.index), e.index))  # errors sorted by LevelOrder
         return True, []
 
     def _is_correct_premise_node(self, node, inference):
@@ -724,7 +746,7 @@ class TableauxSystem:
         return premises, conclusions
 
     def _all_premises_present(self, inference, present_premises, present_conclusions, return_error_list, error_list,
-                              parser):
+                              exit_on_first_error, parser):
         """After visiting all nodes, check that all premises and negated conclusions are present"""
         not_present_premises = set(range(len(inference.premises))) - present_premises
         not_present_conclusions = set(range(len(inference.conclusions))) - present_conclusions
@@ -735,7 +757,10 @@ class TableauxSystem:
                 prem = inference.premises[not_present_premise_idx]
                 if parser:
                     prem = parser.unparse(prem)
-                error_list.append((tuple(), f'Premise {prem} is not present in the tree'))
+                error_list.append(CorrectionError(code=ErrorCode.TBL_PREMISE_NOT_PRESENT, category="TBL", index=tuple(),
+                                                  description=f'Premise {prem} is not present in the tree'))
+                if exit_on_first_error:
+                    return False
         if not_present_conclusions:
             if not return_error_list:
                 return False
@@ -743,7 +768,11 @@ class TableauxSystem:
                 concl = inference.conclusions[not_present_conclusion_idx]
                 if parser:
                     concl = parser.unparse(concl)
-                error_list.append((tuple(), f'Negation of conclusion {concl} is not present in the tree'))
+                error_list.append(CorrectionError(code=ErrorCode.TBL_CONCLUSION_NOT_PRESENT, category="TBL",
+                                                  index=tuple(),
+                                                  description=f'Conclusion {concl} is not present in the tree'))
+                if exit_on_first_error:
+                    return False
         return True
 
     def _is_correctly_applied(self, start_node, rule_subtree, correctly_derived_nodes, subst_dict=None):
@@ -931,6 +960,27 @@ class ManyValuedTableauxSystem(TableauxSystem):
         return premises, conclusions
 
 
+class IndexedTableauxSystem(ManyValuedTableauxSystem, TableauxSystem):
+    """Class for propositional indexed tableaux.
+
+    Initializes nodes as the ``ManyValuedTableauxSystem``, and has ``[(A, 1), (A, 0)]`` as a hardcoded closure rule.
+    """
+    fast_node_is_closed_enabled = True
+
+    @staticmethod
+    def _fast_node_is_closed(node):
+        """Checks whether A, 1 and A, 0 are present in the branch"""
+        path = node.path
+        # Basically, build a new list and add one node at a time, checking that its negation is not present
+        # (or if it is a negated sentence, that the formula it negates is not present)
+        new_list = [(path[0].content, path[0].index)]
+        for node2 in path[1:]:
+            if (node2.content, 1 - node2.index) in new_list:
+                return True
+            new_list.append((node2.content, node2.index))
+        return False
+
+
 # ----------------------------------------------------------------------------------------------------------------------
 # ----------------------------------------------------------------------------------------------------------------------
 
